function [ emd, flow, lpsolver_out ] = emd_banks(sig1, sig2, D, alpha)
%EMD Computes EMD [Ljosa et al, 2006] between two signatures with
%    potentially unequal total masses.
%
% Input:
%
%   sig1 - first signature; |sig1| = n
%   sig2 - second signature; |sig2| = m
%   D - ground distance; an n-by-m matrix, D(i,j) - cost of moving
%       a unit of weight from i'th item in sig1 to j'th item in sig2
%   alpha - factor defining the ground distance to/from the bank from/to regular bins
%           (used for penalizing for histogram mass mismatch)
%
% Output:
%
%   dist - a real value representing the EMD between sig1 and sig2
%   flow - optimal flow of mass between signatures
%   lpsolver_out - raw output of the linear solver
%
% Note 1:
%
%    In the paper, the ground distance to/from the bank is defined as
%    alpha = const * max{D_{ij}}. In our implementation, alpha is the const
%    in the previous expression and typically ranges between 0.5 and +\Inf.
%    Values between 0 and 0.5 are possible, yet, metricity is not guaranteed
%    for them.
%
% Note 2:
%
%    Unlike the original EMD, this EMD does not normalize the transportation cost.
%    Thus, if you want to compare the two, divide the result of emd_banks by
%    max(sum(sig1), sum(sig2)), provided the latter is positive.
%
%#ok<*NASGU>

%% Input validation

if(nargin < 4)
    error(['Not enough input arguments. Besides the signatures and the ground ' ...
          'distance, you need to provide \alpha, for which values in the range ' ...
          '[0.5, 1] are typically used.']);
end

if(~isvector(sig1) || ~isvector(sig2) || ~ismatrix(D))
    error('Invalid input.');
end

[n, m] = size(D);
if(size(sig1, 1) ~= n)
    sig1 = sig1';
end
if(size(sig2, 1) ~= m)
    sig2 = sig2';
end
if(size(sig1, 1) ~= n || size(sig2, 1) ~= m)
    error('Signature sizes do not agree with the size of D.');
end

%% Extending signatures and the ground distance with the bank bin

mass1 = sum(sig1);
mass2 = sum(sig2);
mass_mismatch = abs(mass1 - mass2);
bank_dist = alpha * max(max(D));
if(mass_mismatch > eps)
    sig1(end + 1) = mass2;
    sig2(end + 1) = mass1;
    D(n + 1, m + 1) = 0;
    D(n + 1, 1 : m) = bank_dist;
    D(1 : n, m + 1) = bank_dist;
    [n, m] = size(D);
end

%% Definition of objective function and constaints

% Objective function's coefficients
Dex = reshape(D, n * m, 1);

% Inequality-type constraints (supply/demand limits):
A = []; % Since there is no mass mismatch, we have a circulation problem,
b = []; % and, hence, all constaints are equality-type.

% Equality-type constraints (desirable total flow):
Aeq = [ kron(ones(1, m), speye(n, n)); kron(speye(m, m), ones(1, n)) ];
beq = [ sig1; sig2 ];

% Lower bounds (non-negative flow):
lb = zeros(n * m, 1);

%% Computing EMD

% Choosing solver
if(cplex_available())
    solver_name = 'cplexlp';
else
    solver_name = 'linprog';
end

[consout, flow, emd, exitcode, output] = evalc([solver_name, '(Dex, A, b, Aeq, beq, lb)']);

% Logging
lpsolver_out = struct('consout', consout, 'exitcode', exitcode, 'output', output);

% Preparing output
flow = reshape(flow, n, m);

end