function [ model ] = snd_simple_cost_model(params)
%SND_SIMPLE_COST_MODEL Defines costs for opinion propagation by SND. The model
% is simple in that (1) the costs are defined per user (not per link), and (2)
% the out-costs (for spreading) and the in-costs (for adoption) are the same.
%
% params -- a struct containing parameters of the model:
%
%       costs -- a 3-by-2 matrix of opinion propagation costs. The rows
%           (ordered from 1 to 3) correspond to the acting user's opinion -- +1, 0, -1;
%           and the columns (ordered from 1 to 2) correspond to the type of
%           opinion being propagated -- +1, -1.
%
%           Example:
%
%           costs = [
%               %  +       -
%                   5     100  % +
%                  20      20  % N
%                 100       5  % -
%           ];

    if(nargin < 1)
        params = NaN;
    end

    model = struct('params', params, 'func', @snd_simple_cost_model_func);
end

function [ Aext ] = snd_simple_cost_model_func(model_params, adjmat, user_states, op, transpose)
%SND_SIMPLE_COST_MODEL_FUNC Computes costs for opinion propagation by SND under
%   the simple cost model.
%
% OUTPUT:
%
%   Aext -- matrix of opinion propagation costs
%
% INPUT:
%
%   model_params -- a struct containing parameters of the model:
%
%       costs -- a 3-by-2 matrix of opinion propagation costs. The rows
%           (ordered from 1 to 3) correspond to the acting user's opinion -- +1, 0, -1;
%           and the columns (ordered from 1 to 2) correspond to the type of
%           opinion being propagated -- +1, -1.
%
%           Example:
%
%           costs = [
%               %  +       -
%                   5     100  % +
%                  20      20  % N
%                 100       5  % -
%           ];
%
%   adjmat -- an n-by-n adjacency matrix of the network.
%
%   user_states -- an n-item vector of user states ({-1, 0, +1}^n), based
%       on which opinion propagation costs are computed.
%
%   op -- opinion being propagated ({-1, +1}). If op = +1, then positive users
%       will charge small cost for its propagation, while negative users
%       will charge a lot, and vice versa.
%
%   transpose -- indicated whether to return the original cost matrices or their
%       transposed versions (SND may need to work with either)

%% Input validation

if(nargin < 5)
    error('Not enough arguments.');
end

% Model parameters
if(~isstruct(model_params))
    if(~isnan(model_params))
        error('Invalid model parameters.');
    else
        model_params = struct();
    end
end
if(~isfield(model_params, 'costs'))
    model_params.costs = [
         5, 100;
        20,  20;
       100,   5
    ];
elseif(~isnumeric(model_params.costs) || ~isequal([3, 2], size(model_params.costs)) ...
    || any(any(model_params.costs < 0 | isinf(model_params.costs))))
    error('Invalid model costs.');
end

% Adjacency matrix
if(~isnumeric(adjmat) || size(adjmat, 1) ~= size(adjmat, 2))
    error('Invalid adjacency matrix.');
end

% User states
if(~isvector(user_states) || ~isnumeric(user_states) || length(user_states) ~= size(adjmat, 1))
    error('Invalid vector of user states.');
end

% Opinion being propagated
if(op ~= -1 && op ~= +1)
    error('Invalid opinion being propagated.');
end

% Transpose indicator
if(~islogical(transpose))
    error('Ivalid value of the transpose indicator.');
end

%% Computing costs

costfun = @(sent_type, state) ...
    model_params.costs( ...
        sign(state) + 2, ...
        (sign(sent_type) + 1) / 2 + 1 ...
    );
costs = costfun(op, user_states);

[irows, icols, vals] = find(adjmat);
for k = 1 : length(icols)
    vals(k) = costs(irows(k)) + vals(k) + costs(icols(k));
end
n = length(user_states);
if(~transpose)
    Aext = sparse(irows, icols, vals, n, n, length(vals));
else
    Aext = sparse(icols, irows, vals, n, n, length(vals));
end

end

