function [ cost, flow, log ] = bp_mincost_flow(supplies, demands, D)
%BP_MINCOST_FLOW Slow implementation of min-cost flow for bipartite (BP) graphs
%
% Input:
%
%   supplies -- capacities of the source nodes (left part of the BP graph)
%
%   demands -- capacities of the destination nodes (right part of the BP graph)
%
%   D -- ground distance (rows -- sources, columns -- destinations)
%
% Output:
%
%   cost -- the cost of the optimal flow
%
%   flow -- optimal flow matrix
%
%   log -- diagnostic information from the linear solver (e.g., the reason for
%          misconvergence)
%
% Supressed warnings:
%
%#ok<*NASGU> % unused variables -- uses are not visible outside evalc

eps = 1e-08;

n = length(supplies);
m = length(demands);
nvars = n * m;

%% Input validation
if(nargin < 3)
    error('Not enough arguments.');
end
if(~isvector(supplies) || ~isvector(demands))
    error('Either supplies or/and demands is not a vector.');
end
if(n < 0 || m < 0)
    error('Invalid number of suppliers and/or consumers.');
end
if(n == 0 || m == 0)
    cost = 0;
    flow = zeros(n, m);
    log = 'Degenerate case: empty flow, cost is 0.';
    return;
end
if(size(supplies, 1) < size(supplies, 2))
    supplies = supplies';
end
if(size(demands, 1) < size(demands, 2))
    demands = demands';
end
if(norm(size(D) - [n, m], 1) ~= 0)
    error('Invalid size of the ground distance matrix.');
end
if(abs(sum(supplies) - sum(demands)) > eps)
    error('Total masses of supply and demand vectors are supposed to be equal.');
end

%% Optimization

% Inequality-type constraints
A_ineq = [ ];
b_ineq = [ ];

% Equality-type constraints
A_eq = [
    kron(ones(1, m), speye(n, n)); % row-sum (supply-based) constraints
    kron(speye(m, m), ones(1, n))  % column-sum (demand-based) constraints
];
b_eq = sparse([ supplies; demands ]);

% Lower bounds
lb = zeros(nvars, 1);

% Objective function's coefficients
f = reshape(D, nvars, 1);

% If CPLEX is not available, replace cplexlp with linprog
if(cplex_available)
    lpsolver_name = 'cplexlp';
else
    lpsolver_name = 'linprog';
end
[consout, flow, cost, exitcode, output] = evalc([lpsolver_name, '(f, A_ineq, b_ineq, A_eq, b_eq, lb, [], [])']);

%% Preparing output

flow = reshape(flow, n, m);
log = struct('exitcode', exitcode, 'output', output, 'consout', consout);

if(exitcode ~= 1)
    error('Unable to compute the distance value; linear solver has failed.');
end

end

