function [ cost, flow ] = gt_transport(L, R, D, scale_factor, precision, skip_validation, debug_mode)
%GT_TRANSPORT Solves a balanced (sum(from[i]) = sum(to[j])) transportation problem
% from suppliers L to consumers R at costs D using successive approximation minimum-cost
% flow algorithm by Goldberg and Tarjan [1]. Internally, uses solver CS2 due to Goldberg
% and Cherkassky [2,3].
%
% Input:
%
%   L -- vector of supplies; |L| = n
%
%   R -- vector of demands; |R| = m
%
%   D -- n-by-m ground distance matrix
%
% Options:
%
%   scale_factor -- scale factor (see [1]); default used by CS2 is 12.0
%
%   precision.cost -- number of decimal places to keep for costs in D (is
%                     not important if all D(*, *) are integral)
%
%   precision.cap -- similar precision, but this one is for node capacities. 
%
% Output:
%
%   cost -- cost of the optimal flow / transportation plan
%
% References:
%
%   [1] Goldberg, Tarjan. "Solving minimum-cost flow problems by successive
%                          approximation." ACM, 1987
%
%   [2] Goldberg, "An efficient implementation of a scaling minimum-cost flow
%                  algorithm." Journal of Algorithms 22.1 (1997): 1-29.
%
%   [3] http://www.igsystems.com/cs2/download.html
%       -or-
%       https://web.archive.org/web/20110915040846/http://www.igsystems.com/cs2/download.html
%

%% Validation

if(nargin < 6 || ~islogical(skip_validation))
    skip_validation = false;
end

if(nargin < 7)
    debug_mode = double(false);
else
    debug_mode = double(debug_mode);
end

%% Input validation
if(nargin < 6 || islogical(skip_validation) && ~skip_validation)
    if(nargin < 3)
        error([ ...
            'Not enough arguments. ', ...
            'Supplies (L), demands (R), and ground distance (D) should be provided.' ...
        ]);
    end
    if(any(L < 0) || any(R < 0) || any(any(D < 0)))
        error('Input values must be non-negative.');
    end
    if(~isvector(L) || ~isvector(R))
        error('Both L and R must be vectors.');
    end
    if(issparse(L) || issparse(R) || issparse(D))
        error('Either L or R or D is sparse; full inputs are expected.');
    end
end

%% CS2 check
if(exist('cs2.exe', 'file') ~= 2)
    if(exist('cs2.c', 'file') == 2)
        warning('CS2 binary is not found. Trying to compile it.');    
        cs2_dirpath = fileparts(which('cs2.c'));
        if(prompt_agree_to_license('CS2', fullfile(cs2_dirpath, 'COPYRIGHT.txt')))
            cs2_comp_cmd = sprintf('cd "%s" && make', cs2_dirpath);
            unix(cs2_comp_cmd);
            if(exist('cs2.exe', 'file') ~= 2)
                warning('CS2 compilation failed.');
            else
                warning('CS2 has been successfully compiled.');
            end
        else
            warning('Unable to compile CS2; license has not been accepted.');
            cost = -1;
            flow = -1;
            return;
        end
    else
       warning('CS2 binary is not found. It is source code is also not on MATLAB''s path.'); 
    end
end

%% MEX check
if(exist('gt_transport_mex', 'file') ~= 3)
    compiling = false;
    if(exist('gt_transport_mex_make.m', 'file') == 2)
        fprintf(2, 'gt_transport_mex.mex* file was not found. Trying to compile it.\n');
        compiling = true;
        gt_transport_mex_make;
    end
    if(exist('gt_transport_mex', 'file') ~= 3)
        error([
            'Accompanying gt_transport_mex.mex* file is not found. ', ...
            'The attempt to compile it has probably failed. ', ...
            'Try to compile it manually.'
        ]);
    elseif(compiling)
        fprintf(2, 'gt_transport_mex.mex* has been successfully compiled.\n');
    end
end

if(size(L, 1) < size(L, 2))
    L = L';
end
if(size(R, 1) < size(R, 2))
    R = R';
end
if(nargin < 4 || isnan(scale_factor))
    scale_factor = -1;
end
if(nargin < 5 || ~isstruct(precision))
    precision = struct();
end
if(~isfield(precision, 'cost'))
    precision.cost = 0;
end
if(~isfield(precision, 'cap'))
    precision.cap = 5;
end

n = length(L);
m = length(R);

%% Making costs integer
[ D, cost_mult ] = scale2int(D, precision.cost, debug_mode);
if(debug_mode)
    fprintf('Costs scaled with mult = %f (max cost = %f)\n', full(cost_mult), full(max(max(D))));
end

%% Making node capacities integer
[ L, L_mult ] = scale2int(L, precision.cap, debug_mode);
[ R, R_mult ] = scale2int(R, precision.cap, debug_mode);

cap_mult = max(L_mult, R_mult);
if(L_mult < cap_mult)
    L = L * (cap_mult / L_mult);
elseif(R_mult < cap_mult)
    R = R * (cap_mult / R_mult);
end
clear L_mult R_mult;
if(debug_mode)
    fprintf('Capacities scaled with mult = %f (max cap = %f)\n', full(cap_mult), full(max(max(L), max(R))));
end

mass_mismatch = sum(L) - sum(R);
if(mass_mismatch < 0)
    L(n + 1) = -mass_mismatch;
    D(n + 1, :) = ones(1, m); % zeros
elseif(mass_mismatch > 0)
    R(m + 1) = mass_mismatch;
    D(:, m + 1) = ones(1, n); % zeros
end

if(abs(sum(L) - sum(R)) > eps)
    error('Unable to compensate for the mass mismatch due to the loss of precision.');
elseif(debug_mode)
    fprintf('Successfully compensated for mass mismatch due to the loss of precision.\n');
end

%% Solving transportation problem
if(nargout > 1)
    [ cost, flow_raw ] = gt_transport_mex(L, R, D, scale_factor, debug_mode);
    flow = sparse(flow_raw(:, 1), flow_raw(:, 2) - n, flow_raw(:, 3));
    if(mass_mismatch < 0)
        flow = flow(1 : end - 1, :);
    elseif(mass_mismatch > 0)
        flow = flow(:, 1 : end - 1);
    end
else
    cost = gt_transport_mex(L, R, D, scale_factor, debug_mode);
end

if(cost < 0)
    warning('CS2 was unable to find the solution.');
end

%% Recovering original optimal cost (and flow)
cost = cost / (cost_mult * cap_mult);
if(nargout > 1)
    flow = flow / cap_mult;
end

end

function [ M_int, scale, loss ] = scale2int(M, precision, verbose)
%SCALE2INT Scales values of matrix M by at most 10^precision in order to
% make all values of M integral.
%
% Input:
%
%   M -- matrix to become integer
%
%   precision -- number of digits after the decimal point in the original
%                values to preserve. For example, if M(1,2) = 3.1415, and
%                precision = 2, then, M_int(1,2) will be 314.
%
% Output:
%
%   scale -- M_int(i, j) = int(scale * M(i, j)), for all legal i and j
%
%   loss -- an optional parameter, whose elements are deviations of the
%       scaled values (M_int) from the original ones (M), at the scale of
%       the later.
%
% Example:
%
%   Input:  M     = [ 1.1, 2.05, 1.2345 ], precision = 2
%   Output: M_int = [ 110, 205, 123 ], scale = 100, loss = [ 0, 0, -0.0045 ]
%
%   Input:  M     = [ 1.1, 2 ], precision = 2
%   Output: M_int = [ 11, 20 ], scale = 10, loss = [ 0, 0 ]
%

if(nargin < 3)
    verbose = false;
end

M_int = M;

scale = 1;
for i = 0 : precision - 1
    if(isint(M_int)); break; end;
    M_int = 10 * M_int;
    scale = 10 * scale;
end

if(~isint(M_int))
    if(verbose)
        warning('Lossy rounding with requested precision %d.', precision);
    end
    M_int = round(M_int);
end

if(nargout >= 3)
    loss = M_int ./ scale - M;
end

end

function is = isint(A)
%%ISINT(A) Tells whether matrix A of doubles actually contains only integers
    a = abs(A);
    is = all(a - floor(a) < eps);
end
