function [ arr_mapped, ind_to_val, val_to_ind ] = map_values(arr)
%MAP_VALUES Maps matrix' values to indices in an array of values
%
%    [ arr_mapped, ind_to_val, val_to_ind ] = map_values(arr)
%
% Replaces positive values in a matrix with indices of these values
% is a value array. Returns the index-to-value and the reverse
% value-to-index indices. NaN, +Inf, and negative values are zeroed.
%
% Example:
%
% map_values([10, 0, Inf; 3, 3, 10; NaN NaN 3])
%
%   arr_mapped = [
%       2   0   0
%       1   1   2
%       0   0   1
%   ]
%
%   ind_to_val = [ 3, 10 ]
%
%   val_to_ind = map(keys: [0, 3, 10], values: [0, 1, 2])
%
% Anonymized

    [nrows, ncols] = size(arr);
    if(min(nrows, ncols) <= 0)
        ind_to_val = [];
        val_to_ind = containers.Map('KeyType', 'uint64', 'ValueType', 'uint64');
        arr_mapped = arr;
        return;
    end

    arr(arr >= Inf) = 0;
    arr(isnan(arr)) = 0;
    arr(arr < 0) = 0;

    ind_to_val = unique(arr(arr > 0));
    n_unique_values = size(ind_to_val, 1);
    inds = (1:n_unique_values)';
    val_to_ind = containers.Map([0; ind_to_val], [0; inds], 'UniformValues', true);
    
    % MATLAB's dedicated function changem requires Mapping Toolbox
    arr_mapped = zeros([nrows, ncols]);
    parfor i = 1 : nrows
        for j = 1 : ncols
            arr_mapped(i, j) = val_to_ind(arr(i, j));
        end
    end
end
